import os
import datetime
import json
import logging
import tempfile
from poster.encode import multipart_encode
from poster.streaminghttp import register_openers
import urllib2
from subprocess import Popen, PIPE

logger = logging.getLogger(__name__)


class Report(object):

    def __init__(self, machine_id, name, api_root):
        self.id = machine_id
        self.name = name
        (file_fd, self.file_path) = tempfile.mkstemp(suffix=self.name + ".json")
        self.file = os.fdopen(file_fd, "w+")
        self.url = api_root + "/".join(self.name.split("_")) + "/"
        # Data that will be send to the API in JSON format
        self.data = {}
        self.send_file = False
        logger.info("%s running" % self.name)

    def run(self):
        raise Exception("Must be implemented by subclass")

    def send(self):
        if not self.data and not self.send_file:
            logger.warning("No data for %s report. Skipping" % self.name)
            return
        # Always add the machine id to the request
        self.data["machine"] = self.id
        logger.debug("Sending %s data to %s" % (self.name, self.url))
        register_openers()

        if self.send_file:
            logger.debug("File is %s" % self.file_path)
            self.file.close()
            self.data["file"] = open(self.file_path, "rb")
            data, headers = multipart_encode(self.data)
        else:
            dthandler = lambda obj: obj.isoformat() if isinstance(obj, datetime.datetime) else None
            data = json.dumps(self.data, default=dthandler)
            headers = {'Content-Type': 'application/json'}

        try:
            r = urllib2.Request(self.url, data, headers)
            logger.debug(urllib2.urlopen(r).read())
        except Exception as e:
            logger.error(e)

        logger.debug("Deleting temporary file...")
        os.unlink(self.file_path)
        logger.info("Done!")


def execute(command, cwd=None, shell=False, env=None):
    p = Popen(command, stdout=PIPE, stderr=PIPE, bufsize=1, cwd=cwd,
              shell=shell, env=env)
    (stdout, stderr) = p.communicate()

    return (p.returncode, stdout, stderr)
