#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

from mbs.reports.client import Report, execute
from json import loads as json_parse

logger = logging.getLogger(__name__)


class PulseSubscriptionReport(Report):

    def run(self):
        # Get registered computers
        (code, stdout, stderr) = execute(["/usr/sbin/pulse2-debug", "-c", "print base.getComputerCount({})"])
        if code == 0:
            registered_computers = int(stdout)
        else:
            raise Exception("Failed to retrieve registered computers: %s" % stderr)

        # Get max computers available
        (code, stdout, stderr) = execute(["/usr/sbin/pulse-licensed", "-G", "/etc/pulse-licensing/installation_id", "-l", "/etc/pulse-licensing/license.dat", "-p", "/etc/pulse-licensing/license", "-i"])
        if code == 0:
            datas = json_parse(stdout)
            max_computers = 5
            for data in datas:
                if 'alias' in data and 'actif' in data:
                    if data['alias'] == 'pulse' and data['actif'] == 'yes':
                        # because of a strange API made by a strange guy, max_computers is stored in 'custon_number' key
                        # [16:12] <acecile> pourquoi ça s'appelle custon number -_-'
                        # [16:13] <jfkneib> oui j'ai peut être pas bien choisie le nom il y a confusion.
                        max_computers = int(data['custon_number'])
                        break
        else:
            raise Exception("Failed to retrieve max computers license: %s" % stderr)

        # Get installation_id (in case of demo license, pulse-licensed returns an empty list, without installation_id)
        try:
            installation_id = open("/etc/pulse-licensing/installation_id").read().strip()
        except IOError:
            raise Exception("Failed to retrieve Pulse installation_id")

        # Get active modules
        (code, stdout, stderr) = execute(["/usr/sbin/pulse2-debug", "-c", "print list(set([x.split('.')[0] for x in system.listMethods()]))"])
        if code == 0:
            stdout = eval(stdout)
            stdout.sort()
            active_modules = ', '.join(stdout)
        else:
            raise Exception("Failed to retrieve modules list: %s" % stderr)

        self.data = {
            'registered_computers': registered_computers,
            'max_computers': max_computers,
            'active_modules': active_modules,
            'installation_id': installation_id,
        }
